/* SPDX-License-Identifier: GPL-2.0+ OR BSD-3-Clause */
/*
 * Copyright (C) 2018, STMicroelectronics - All Rights Reserved
 *
 * Configuration settings for the STM32MP15x CPU
 */

#ifndef __CONFIG_H
#define __CONFIG_H
#include <linux/sizes.h>
#include <asm/arch/stm32.h>

/*
 * Number of clock ticks in 1 sec
 */
#define CONFIG_SYS_HZ				1000

#ifndef CONFIG_STM32MP1_TRUSTED
/* PSCI support */
#define CONFIG_ARMV7_PSCI_1_0
#define CONFIG_ARMV7_SECURE_BASE		STM32_SYSRAM_BASE
#define CONFIG_ARMV7_SECURE_MAX_SIZE		STM32_SYSRAM_SIZE
#endif

/*
 * malloc() pool size
 */
#define CONFIG_SYS_MALLOC_LEN			SZ_32M

/*
 * Configuration of the external SRAM memory used by U-Boot
 */
#define CONFIG_SYS_SDRAM_BASE			STM32_DDR_BASE
#define CONFIG_SYS_INIT_SP_ADDR			CONFIG_SYS_TEXT_BASE

#define CONFIG_DISABLE_CONSOLE

/*
 * Console I/O buffer size
 */
#define CONFIG_SYS_CBSIZE			SZ_1K

/*
 * Needed by "loadb"
 */
#define CONFIG_SYS_LOAD_ADDR			STM32_DDR_BASE

/*
 * Env parameters
 */
#define CONFIG_ENV_OVERWRITE
#define CONFIG_ENV_SIZE				SZ_16K

/* Environment in SPI NOR */
#ifdef CONFIG_ENV_IS_IN_SPI_FLASH

#define CONFIG_ENV_OFFSET		(0x18 * SZ_64K)
#define CONFIG_ENV_SECT_SIZE	(SZ_64K)
#define CONFIG_ENV_SPI_BUS		(0)
#define CONFIG_ENV_SPI_CS		(0)
#define CONFIG_ENV_SPI_MODE 	(SPI_RX_QUAD | SPI_TX_QUAD)
#define CONFIG_ENV_SPI_MAX_HZ	(8000000)

#endif

// For sd card boot
#ifdef CONFIG_ENV_IS_IN_MMC

#ifndef CONFIG_SYS_MMC_ENV_DEV
#define CONFIG_SYS_MMC_ENV_DEV  (0)
#endif

#ifndef CONFIG_SYS_MMC_ENV_PART
#define CONFIG_SYS_MMC_ENV_PART (0)
#endif

// fsbl1 (0x40000), fsbl2 (0x40000), ssbl (0x200000)
#define CONFIG_ENV_OFFSET		(0x280000-CONFIG_ENV_SIZE)
#endif

/* ATAGs */
#define CONFIG_CMDLINE_TAG
#define CONFIG_SETUP_MEMORY_TAGS
#define CONFIG_INITRD_TAG

/* Extend size of kernel image for uncompression */
#define CONFIG_SYS_BOOTM_LEN			SZ_32M

/* SPL support */
#ifdef CONFIG_SPL
/* BOOTROM load address */
#define CONFIG_SPL_TEXT_BASE		0x2FFC2500
/* SPL use DDR */
#define CONFIG_SPL_BSS_START_ADDR	0xC0200000
#define CONFIG_SPL_BSS_MAX_SIZE		0x00100000
#define CONFIG_SYS_SPL_MALLOC_START	0xC0300000
#define CONFIG_SYS_SPL_MALLOC_SIZE	0x00100000

/* limit SYSRAM usage to first 128 KB */
#define CONFIG_SPL_MAX_SIZE		0x00020000
#define CONFIG_SPL_STACK		(STM32_SYSRAM_BASE + \
					 STM32_SYSRAM_SIZE)
#endif /* #ifdef CONFIG_SPL */

#define CONFIG_SYS_MEMTEST_START	STM32_DDR_BASE
#define CONFIG_SYS_MEMTEST_END		(CONFIG_SYS_MEMTEST_START + SZ_64M)
#define CONFIG_SYS_MEMTEST_SCRATCH	(CONFIG_SYS_MEMTEST_END + 4)

/*MMC SD*/
#define CONFIG_SYS_MMC_MAX_DEVICE	3
#define CONFIG_SUPPORT_EMMC_BOOT

/*
 * define serial used for u-boot
 *
 * */
#define CONFIG_CONS_INDEX 2

/*****************************************************************************/
#ifdef CONFIG_DISTRO_DEFAULTS
/*****************************************************************************/

/* NAND support */
#define CONFIG_SYS_NAND_ONFI_DETECTION
#define CONFIG_SYS_MAX_NAND_DEVICE	1

/* SPI nand */
#define CONFIG_SYS_MAX_NAND_DEVICE	1

/* SPI FLASH support */
#if defined(CONFIG_SPL_BUILD)
#define CONFIG_SYS_SPI_U_BOOT_OFFS	0x80000
#endif

/* FILE SYSTEM */

#if defined(CONFIG_STM32_QSPI) || defined(CONFIG_NAND_STM32_FMC2)
/* Dynamic MTD partition support */
#define CONFIG_SYS_MTDPARTS_RUNTIME
#endif

/* Ethernet need */
#ifdef CONFIG_DWC_ETH_QOS
#define CONFIG_SYS_NONCACHED_MEMORY	(1 * SZ_1M)	/* 1M */
#define CONFIG_SERVERIP			192.168.1.240
#define CONFIG_BOOTP_SERVERIP
#define CONFIG_SYS_AUTOLOAD		"no"
#endif

#ifdef CONFIG_DM_VIDEO
#define CONFIG_VIDEO_BMP_RLE8
#define CONFIG_BMP_16BPP
#define CONFIG_BMP_24BPP
#define CONFIG_BMP_32BPP
#endif

#if !defined(CONFIG_SPL_BUILD)

/* default order is SDCARD (SDMMC 0) / eMMC (SDMMC 1) / NAND /  */
#define BOOT_TARGET_DEVICES(func) \
	func(MMC, mmc, 0) \
	func(MMC, mmc, 1) \
	func(UBIFS, ubifs, 0) \
	func(PXE, pxe, na)

#include <config_distro_bootcmd.h>

#if defined(CONFIG_CMD_UBIFS) && defined(CONFIG_SPLASH_SCREEN)
#undef BOOTENV_SHARED_UBIFS
#define BOOTENV_SHARED_UBIFS \
	"ubifs_boot=" \
		"if test ${boot_device} = nand; then " \
			"env exists bootubivol || " \
				"env set bootubivol boroot; " \
			"if ubifsmount ubi${devnum}:${bootubivol}; " \
			"then " \
				"setenv devtype ubi; " \
				"run scan_dev_for_boot; " \
			"fi;" \
		"else " \
			"env exists bootubipart || " \
				"env set bootubipart UBI; " \
			"env exists bootubivol || " \
				"env set bootubivol boroot; " \
			"if ubi part ${bootubipart} && " \
				"ubifsmount ubi${devnum}:${bootubivol}; " \
			"then " \
				"setenv devtype ubi; " \
				"run scan_dev_for_boot; " \
			"fi;" \
		"fi\0"
#endif

#define CONFIG_PREBOOT \
	"dcache off; setenv board_name ${board_name}${housing}; " \
	"echo \"Boot over ${boot_device}${boot_instance}!\"; " \
	"if test ${boot_device} = uart; then " \
		"stm32prog serial ${boot_instance}; " \
	"else "\
		"if test ${boot_device} = usb; then " \
			"stm32prog usb ${boot_instance}; " \
		"fi; "\
	"fi; "

#ifdef CONFIG_STM32MP1_OPTEE
#define CONFIG_SYS_MEM_TOP_HIDE			SZ_32M
/* with OPTEE: define specific MTD partitions = teeh, teed, teex */
#define STM32MP_MTDPARTS \
	"mtdparts_nor0=256k(fsbl1),256k(fsbl2),2m(ssbl),256k(logo),256k(teeh),256k(teed),256k(teex),-(nor_user)\0" \
	"mtdparts_spi-nand0=-(UBI);\0"


#else /* CONFIG_STM32MP1_OPTEE */
#define STM32MP_MTDPARTS \
	"mtdparts_nor0=256k(fsbl1),256k(fsbl2),1m(ssbl),64k(env),-(nor_user)\0" \
	"mtdparts_spi-nand0=-(UBI)\0"

#endif /* CONFIG_STM32MP1_OPTEE */

/*u-boot vars needed for rescue os boot*/
#define RESCUE_OS_ENV \
	"res_kernel_img= uImage \0" \
	"res_os_img= rescue-os.cpio.gz.u-boot \0" \
	"res_dt= stm32mp-t1000-s-50.dtb \0" \
	"rescue_os_flag=0\0" \
	"upgrade_available=0\0" \
	"altbootcmd=run bootcmd\0" \
	"boot_rescue_os=" \
		"for target in ${boot_targets}; do " \
			"run boot_res_${target}; " \
		"done\0" \
	"boot_res_mmc0= " \
		"env set devnum 0;" \
		"env set res_part 3;" \
		"run boot_res_mmcx;\0" \
	"boot_res_mmc1= " \
		"env set devnum 1;" \
		"env set res_part 3;" \
		"run boot_res_mmcx;\0" \
	"boot_res_mmcx= " \
		"if mmc dev ${devnum};" \
		"then " \
			"load mmc ${devnum}:${res_part} ${kernel_addr_r} ${res_kernel_img} ;" \
			"load mmc ${devnum}:${res_part} ${ramdisk_addr_r} ${res_os_img} ;" \
			"load mmc ${devnum}:${res_part} ${fdt_addr_r} ${res_dt};" \
			"bootm ${kernel_addr_r} ${ramdisk_addr_r} ${fdt_addr_r};" \
		"fi;\0" \
	"boot_res_ubifs0= " \
		"if ubi part ubi || ubi part UBI ;" \
		"then " \
			"ubifsmount ubi0:rescue-os; " \
			"ubifsload ${kernel_addr_r} ${res_kernel_img} ; " \
			"ubifsload ${ramdisk_addr_r} ${res_os_img} ;" \
			"ubifsload ${fdt_addr_r} ${res_dt};" \
			"bootm ${kernel_addr_r} ${ramdisk_addr_r} ${fdt_addr_r};" \
		"fi;\0" \
	"bootcmd=" \
		"if test ${rescue_os_flag} = 1; then " \
			"run boot_rescue_os;" \
		"else " \
			"run distro_bootcmd;" \
		"fi\0" \
	"altbootcmd=" \
	    "echo ===========================;" \
	    "echo WARNING: RESCUE BOOT ACTIVE;" \
	    "echo ===========================;" \
		"env set rescue_os_flag 1;" \
		"run bootcmd;\0"\

/*
 * memory layout for 32M uncompressed/compressed kernel,
 * 1M fdt, 1M script, 1M pxe and 1M for splashimage
 * and the ramdisk at the end.
 */
#define CONFIG_EXTRA_ENV_SETTINGS \
	"stdin=serial\0" \
	"stdout=serial\0" \
	"stderr=serial\0" \
	"kernel_addr_r=0xc2000000\0" \
	"fdt_addr_r=0xc4000000\0" \
	"scriptaddr=0xc4100000\0" \
	"pxefile_addr_r=0xc4200000\0" \
	"splashimage=0xc4300000\0"  \
	"ramdisk_addr_r=0xc4400000\0" \
	"fdt_high=0xffffffff\0" \
	"initrd_high=0xffffffff\0" \
	"usb_pgood_delay=2000\0" \
	STM32MP_MTDPARTS \
	BOOTENV \
	"boot_net_usb_start=true\0" \
	"housing=" CONFIG_HOUSING_DEFAULT "\0" \
	RESCUE_OS_ENV

#endif /* ifndef CONFIG_SPL_BUILD */
#endif /* ifdef CONFIG_DISTRO_DEFAULTS*/

#endif /* __CONFIG_H */
